% *************************************************************************
% **************************** The OEOA Toolbox ***************************
% The On-line Ensemble Using Ordered Aggregation (OEOA) Toolbox
% *************************************************************************
% An Adaptive Ensemble of On-line Extreme Learning Machines with Variable 
% Forgetting Factor for Dynamic System Prediction
% *************************************************************************
% Version 1.0 Nov/27/2015
%
% Developed in MATLAB R2010a
% 
% Author and programmer: Symone Gomes Soares
%
% e-mail: symonesoares@gmail.com 
%
% Homepage: http://sites.google.com/site/symonesoares/
%   
% Paper: SOARES, Symone Gomes; ARAÚJO, Rui. An Adaptive Ensemble of On-line 
% Extreme Learning Machines with Variable Forgetting Factor for Dynamic 
% System Prediction. Neurocomputing, v. 171, p. 693-707, 2016. 
%        DOI: http://dx.doi.org/10.1016/j.neucom.2015.07.035
%
% *************************************************************************
% *************************************************************************
% *************************************************************************
% Algorithm Description:
%    The training of the OS-ELM algorithm for regression 
% *************************************************************************
% Inputs of the Algorithm:
%
% - P0(number of samples x number of input variables) : input matrix
% 
% - T0(number of samples x output variable): output matrix 
%   ps.: it accepts only one output variable
%
% - nHiddenNeurons: number of neurons in the hidden layer
%                   ps.: nHiddenNeurons >= number of samples
%
% - ActivationFunction: activation function for OS-ELM
%                       Options: 'rbf'; 'sig'; 'sin'; 'hardlim';
%                       Default: 'sig';
% 
% *************************************************************************
% Outputs of the Algorithm:
%
% - TrainAcc: accuracy of the model in the training; it includes correlation 
%             coefficient, mean square error, and root mean square error
%
% - TrainOutput: training outputs of the OS-ELM model
%
% - ELM: parameters of the OS-ELM model
%
% *************************************************************************
% *************************************************************************
% *************************************************************************

function [TrainAcc, TrainOutput, ELM] = OSELM_Reg_Train(P0, T0, nHiddenNeurons, ActivationFunction)

    %%%%%%%%%%% Load dataset
    nInputNeurons=size(P0,2);

    %%%%%%%%%%% step 1 Initialization Phase
    IW = rand(nHiddenNeurons,nInputNeurons)*2-1;
    switch lower(ActivationFunction)
        case{'rbf'}
            Bias = rand(1,nHiddenNeurons);
            H0 = RBFun(P0,IW,Bias);
        case{'sig'}
            Bias = rand(1,nHiddenNeurons)*2-1;
            H0 = SigActFun(P0,IW,Bias);
        case{'sin'}
            Bias = rand(1,nHiddenNeurons)*2-1;
            H0 = SinActFun(P0,IW,Bias);
        case{'hardlim'}
            Bias = rand(1,nHiddenNeurons)*2-1;
            H0 = HardlimActFun(P0,IW,Bias);
            H0 = double(H0);
    end
    M    = pinv(H0' * H0);
    beta = pinv(H0) * T0;
    
    %%%%%%%%%%% Calculate the training accuracy
    Y = H0 * beta;                            %   Y: the actual output of the training data
    TrainAcc.rmse = sqrt(mse(T0 - Y));        %   Calculate training accuracy (RMSE) for regression case
    TrainAcc.mse  = mse(T0 - Y);              %   Calculate training accuracy (MSE) for regression case
    TrainAcc.cc   = corr2(T0, Y);             %   Calculate training accuracy (CC) for regression case
    TrainOutput   = Y;
    clear H;
    
    %Giving the ELM parameters
    ELM.NumberofHiddenNeurons = nHiddenNeurons;
    ELM.ActivationFunction = ActivationFunction;
    ELM.Bias = Bias;
    ELM.beta = beta;
    ELM.IW   = IW;
    ELM.M    = M;
end
 
